import conventional from '@commitlint/config-conventional';
import { execSync } from 'node:child_process';
import { readdirSync, statSync } from 'node:fs';

function getGitDiffLines() {
  try {
    return execSync('git diff --name-only --staged').toString().trim().split('\n');
  } catch {
    return [];
  }
}

function getTypes(dir) {
  try {
    return readdirSync(dir).filter(entity => statSync(`${dir}/${entity}`).isDirectory());
  } catch {
    return [];
  }
}

var index = {
  extends: ['@commitlint/config-conventional'],
  rules: {
    'scope-enum': () => {
      return [2, 'always', ['release', 'deprecate', 'schematics', 'all', 'deps', ...getTypes('projects'), ...getTypes('apps'), ...getTypes('libs'), ...getTypes('packages')]];
    },
    'type-enum': () => {
      const staged = getGitDiffLines();
      const demoChanges = staged.filter(path => path.startsWith('projects/demo') || path.startsWith('apps/demo'));
      const [level, applicable, types] = conventional.rules['type-enum'];
      const prefixes = demoChanges.length === staged.length ? ['chore'] : [...types, 'deprecate'];
      return [level, applicable, prefixes];
    }
  }
};

export { index as default };
